@@ -16,6 +16,11 @@ import glob
 import locale
 import types
 
+if sys.version_info.major == 2:
+    # suport cp65001 aka utf-8
+    import codecs
+    codecs.register(lambda name: codecs.lookup('utf-8') if name == 'cp65001' else None)
+
 try:
     sorted( [] )
 except NameError:
@@ -384,20 +389,48 @@ class SvnCommand:
     def cmd_annotate( self, args ):
         start_revision, end_revision = args.getOptionalRevisionPair( '--revision', '0', 'head' )
         positional_args = args.getPositionalArgs( 1, 1 )
-        all_lines = self.client.annotate( positional_args[0],
-                    revision_start=start_revision,
-                    revision_end=end_revision )
+        all_lines = self.client.annotate( 
+                                    positional_args[0],
+                                    revision_start=start_revision,
+                                    revision_end=end_revision )
         self.printNotifyMessages()
 
         for line in all_lines:
             print( '%d| r%d | %s | %s | %s' %
-                (line['number']
+                (line['number']+1
                 ,line['revision'].number
                 ,line['author']
                 ,line['date']
                 ,line['line']) )
     cmd_ann = cmd_annotate
 
+    def cmd_annotate2( self, args ):
+        if not hasattr( self.client, 'annotate2' ):
+            print( 'annotate2 is not available in this version of subversion' )
+            return
+
+        start_revision, end_revision = args.getOptionalRevisionPair( '--revision', '0', 'head' )
+        positional_args = args.getPositionalArgs( 1, 1 )
+
+        all_lines = self.client.annotate2(
+                                positional_args[0],
+                                revision_start=start_revision,
+                                revision_end=end_revision )
+
+        self.printNotifyMessages()
+
+        for line in all_lines:
+            print( '%d| r%d | %s' %
+                (line['number']+1
+                ,line['revision'].number
+                ,line['line']) )
+            if line['merged_revision'] is not None:
+                print( '   Merged from r%d %s' %
+                    (line['merged_revision']
+                    ,line['merged_path']) )
+
+    cmd_ann2 = cmd_annotate2
+
     def cmd_cat( self, args ):
         revision = args.getOptionalRevision( '--revision', 'head' )
         text = self.client.cat( args.getPositionalArgs( 1, 1 )[0], revision=revision )
@@ -628,12 +661,11 @@ class SvnCommand:
                 if wc_info.checksum:
                     print( 'Checksum: %s' %  wc_info.checksum )
 
-
     def cmd_import( self, args ):
-        msg = self.getOptionalValue( '--message', '' )
+        msg = args.getOptionalValue( '--message', '' )
         recurse = args.getBooleanOption( '--non-recursive', False )
         positional_args = args.getPositionalArgs( 2, 2 )
-        self.client.import_( positional_args[0], positional_args[1], recurse=recurse )
+        self.client.import_( positional_args[0], positional_args[1], msg, recurse=recurse )
 
     def cmd_lock( self, args ):
         msg = args.getOptionalValue( '--message', '' )
@@ -643,12 +675,14 @@ class SvnCommand:
 
     def cmd_log( self, args ):
         start_revision, end_revision = args.getOptionalRevisionPair( '--revision', 'head', '0' )
+        limit = args.getOptionalValue( '--limit', 0 )
         verbose = args.getBooleanOption( '--verbose', True )
         positional_args = args.getPositionalArgs( 1, 1 )
         all_logs = self.client.log( positional_args[0],
                                     revision_start=start_revision,
                                     revision_end=end_revision,
-                                    discover_changed_paths=verbose )
+                                    discover_changed_paths=verbose,
+                                    limit=limit )
 
         for log in all_logs:
             print( '-'*60 )
@@ -683,20 +717,20 @@ class SvnCommand:
             positional_args.append( '.' )
 
         for arg in positional_args:
-            all_files = self.client.ls( arg, revision=revision, recurse=recurse )
-            all_files.sort( key=self.__sortKeyLsList )
+            all_entries = self.client.ls( arg, revision=revision, recurse=recurse )
+            all_entries.sort( key=self.__sortKeyLsList )
             if verbose:
-                for file in all_files:
+                for entry in all_entries:
                     args = {}
-                    args.update( file )
-                    args['name'] = args['name']  
+                    args.update( entry )
                     args['last_author'] = args['last_author']  
-                    args['time_str'] = fmtDateTime( file.time )
-                    args['created_rev_num'] = file.created_rev.number
+                    args['time_str'] = fmtDateTime( entry.time )
+                    args['created_rev_num'] = entry.created_rev.number
                     print( '%(created_rev_num)7d %(last_author)-10s %(size)6d %(time_str)s %(name)s' % args )
+
             else:
-                for file in all_files:
-                    print( '%(name)s' % file )
+                for entry in all_entries:
+                    print( '%(name)s' % entry )
 
     def __sortKeyLsList( self, entry ):
         return entry['name']
@@ -705,22 +739,41 @@ class SvnCommand:
         recurse = args.getBooleanOption( '--recursive', True )
         revision = args.getOptionalRevision( '--revision', 'head' )
         verbose = args.getBooleanOption( '--verbose', True )
+        fetch_locks = args.getBooleanOption( '--fetch-locks', True )
+        include_externals = args.getBooleanOption( '--include-externals', True )
         positional_args = args.getPositionalArgs( 0 )
         if len(positional_args) == 0:
             positional_args.append( '.' )
 
         for arg in positional_args:
-            all_files = self.client.list( arg, revision=revision, recurse=recurse )
-            if verbose:
-                for file, Q in all_files:
-                    args = {}
-                    args.update( file )
-                    args['time_str'] = fmtDateTime( file.time )
-                    args['created_rev_num'] = file.created_rev.number
-                    print( '%(created_rev_num)7d %(last_author)-10s %(size)6d %(time_str)s %(path)s' % args )
+            if self.pysvn_testing >= '01.08.00':
+                all_entries = self.client.list( arg, revision=revision, recurse=recurse, fetch_locks=fetch_locks, include_externals=include_externals )
             else:
-                for file, Q in all_files:
-                    print( '%(path)s' % file )
+                all_entries = self.client.list( arg, revision=revision, recurse=recurse, fetch_locks=fetch_locks )
+
+            if verbose:
+                for entry_tuple in all_entries:
+                    entry = entry_tuple[0]
+                    lock_info = entry_tuple[1]
+                    args = {}
+                    args.update( entry )
+                    args['time_str'] = fmtDateTime( entry.time )
+                    args['created_rev_num'] = entry.created_rev.number
+                    print( '%(created_rev_num)7d %(last_author)-10s %(size)6d %(time_str)s %(path)s' % args )
+                    if lock_info is not None:
+                        print( '        Lock   owner: %s' % (lock_info.owner,) )
+                        print( '        Lock comment: %s' % (lock_info.comment,) )
+                        if lock_info.creation_date is not None:
+                            print( '        Lock created: %s' % (time.strftime( '%Y-%m-%d %H:%M:%S', time.localtime( lock_info.creation_date ) ),) )
+                        if lock_info.expiration_date is not None:
+                            print( '        Lock expires: %s' % (time.strftime( '%Y-%m-%d %H:%M:%S', time.localtime( lock_info.expiration_date ) ),) )
+                    if self.pysvn_testing >= '01.08.00' and include_externals and entry_tuple[2] is not None:
+                        print( '        External target %s' % (entry_tuple[3],) )
+                        print( '        External    URL %s' % (entry_tuple[2],) )
+
+            else:
+                for entry_tuple in all_entries:
+                    print( '%(path)s' % entry_tuple[0] )
 
     def cmd_merge( self, args ):
         recurse = args.getBooleanOption( '--recursive', True )
@@ -816,23 +869,44 @@ class SvnCommand:
     def cmd_propget( self, args ):
         recurse = args.getBooleanOption( '--recursive', True )
         revision = args.getOptionalRevision( '--revision', 'working' )
+        if self.pysvn_testing >= '01.08.00' and args.getBooleanOption( '--show-inherited-props' ):
+            get_inherited_props = True
+        else:
+            get_inherited_props = False
         positional_args = args.getPositionalArgs( 1, 2 )
         if len(positional_args) == 1:
             positional_args.append( '.' )
         if self.client.is_url( positional_args[0] ):
             revision = args.getOptionalRevision( '--revision', 'head' )
 
-        props = self.client.propget( positional_args[0], positional_args[1], revision=revision, recurse=recurse )
+        if get_inherited_props:
+            props, inherited_props = self.client.propget(
+                    positional_args[0], positional_args[1],
+                    revision=revision,
+                    recurse=recurse,
+                    get_inherited_props=True )
+        else:
+            props = self.client.propget(
+                    positional_args[0], positional_args[1],
+                    revision=revision,
+                    recurse=recurse )
+            inherited_props = {}
+
         prop_names = sorted( props.keys() )
         for name in prop_names:
             print( '%s: %s' % (name, props[name]) )
 
+        if len(inherited_props) > 0:
+            print( 'Inherited props' )
+            prop_names = sorted( inherited_props.keys() )
+            for name in prop_names:
+                print( '%s: %s' % (name, inherited_props[name]) )
+
     cmd_pg = cmd_propget
 
     def cmd_propset( self, args ):
         recurse = args.getBooleanOption( '--recursive', True )
         revision = args.getOptionalRevision( '--revision', 'working' )
-        verbose = args.getBooleanOption( '--verbose', True )
         positional_args = args.getPositionalArgs( 2, 3 )
         if len(positional_args) == 2:
             positional_args.append( '.' )
@@ -845,7 +919,6 @@ class SvnCommand:
     def cmd_propdel( self, args ):
         recurse = args.getBooleanOption( '--recursive', True )
         revision = args.getOptionalRevision( '--revision', 'working' )
-        verbose = args.getBooleanOption( '--verbose', True )
         positional_args = args.getPositionalArgs( 1, 2 )
         if len(positional_args) == 1:
             positional_args.append( '.' )
@@ -855,6 +928,46 @@ class SvnCommand:
         self.client.propdel( positional_args[0], positional_args[1], revision=revision, recurse=recurse )
     cmd_pd = cmd_propdel
 
+    def cmd_propset_local( self, args ):
+        skip_checks = args.getBooleanOption( '--skip-checks', True )
+        changelist = args.getOptionalValue( '--change-list', None )
+        positional_args = args.getPositionalArgs( 3, 3 )
+
+        if changelist is not None:
+            changelist = [s.strip() for s in changelist.split(',')]
+            self.client.propset_local( positional_args[0], positional_args[1], positional_args[2], skip_checks=skip_checks, changelist=changelist )
+
+        else:
+            self.client.propset_local( positional_args[0], positional_args[1], positional_args[2], skip_checks=skip_checks )
+
+    def cmd_propdel_local( self, args ):
+        changelist = args.getOptionalValue( '--change-list', None )
+        positional_args = args.getPositionalArgs( 2, 2 )
+
+        if changelist is not None:
+            changelist = [s.strip() for s in changelist.split(',')]
+            self.client.propdel_local( positional_args[0], positional_args[1], changelist=changelist )
+
+        else:
+            self.client.propdel_local( positional_args[0], positional_args[1] )
+
+    def cmd_propset_remote( self, args ):
+        skip_checks = args.getBooleanOption( '--skip-checks', True )
+        revision = args.getOptionalRevision( '--revision', '0' )
+        positional_args = args.getPositionalArgs( 3, 3 )
+
+        if args.haveOption( '--revision' ):
+            self.client.propset_remote( positional_args[0], positional_args[1], positional_args[2], skip_checks=skip_checks, base_revision_for_url=revision )
+
+        else:
+            self.client.propset_remote( positional_args[0], positional_args[1], positional_args[2], skip_checks=skip_checks )
+
+    def cmd_propdel_remote( self, args ):
+        revision = args.getOptionalRevision( '--revision', '0' )
+        positional_args = args.getPositionalArgs( 2, 2 )
+
+        self.client.propdel_remote( positional_args[0], positional_args[1], base_revision_for_url=revision )
+
     def cmd_revproplist( self, args ):
         revision = args.getOptionalRevision( '--revision', 'head' )
         verbose = args.getBooleanOption( '--verbose', False )
@@ -938,52 +1051,52 @@ class SvnCommand:
 
         positional_args = args.getPositionalArgs( 0 )
         if len(positional_args) == 0:
-            all_files = self.client.status( '', recurse=recurse, get_all=verbose, ignore=ignore, update=update )
-            self._cmd_status_print( all_files, verbose, update, ignore, quiet )
+            all_entries = self.client.status( '', recurse=recurse, get_all=verbose, ignore=ignore, update=update )
+            self._cmd_status_print( all_entries, verbose, update, ignore, quiet )
         else:
             for arg in positional_args:
-                all_files = self.client.status( arg, recurse=recurse, get_all=verbose, ignore=ignore, update=update )
-                self._cmd_status_print( all_files, verbose, update, ignore, quiet )
+                all_entries = self.client.status( arg, recurse=recurse, get_all=verbose, ignore=ignore, update=update )
+                self._cmd_status_print( all_entries, verbose, update, ignore, quiet )
 
-    def _cmd_status_print( self, all_files, detailed, update, ignore, quiet ):
-        all_files.sort( key=self.key_by_path )
-        for file in all_files:
-            if file.text_status == pysvn.wc_status_kind.ignored and ignore:
+    def _cmd_status_print( self, all_entries, detailed, update, ignore, quiet ):
+        all_entries.sort( key=self.key_by_path )
+        for entry in all_entries:
+            if entry.text_status == pysvn.wc_status_kind.ignored and ignore:
                 continue
 
-            if file.text_status == pysvn.wc_status_kind.unversioned and quiet:
+            if entry.text_status == pysvn.wc_status_kind.unversioned and quiet:
                 continue
 
-            state = '%s%s%s%s%s' % (wc_status_kind_map[ file.text_status ],
-                    wc_status_kind_map[ file.prop_status ],
-                    ' L'[ file.is_locked ],
-                    ' +'[ file.is_copied ],
-                    ' S'[ file.is_switched ])
+            state = '%s%s%s%s%s' % (wc_status_kind_map[ entry.text_status ],
+                    wc_status_kind_map[ entry.prop_status ],
+                    ' L'[ entry.is_locked ],
+                    ' +'[ entry.is_copied ],
+                    ' S'[ entry.is_switched ])
 
-            if( file.repos_text_status != pysvn.wc_status_kind.none
-            or  file.repos_prop_status != pysvn.wc_status_kind.none ):
-                odd_status = '%s%s' % (wc_status_kind_map[ file.repos_text_status ],
-                    wc_status_kind_map[ file.repos_prop_status ])
+            if( entry.repos_text_status != pysvn.wc_status_kind.none
+            or  entry.repos_prop_status != pysvn.wc_status_kind.none ):
+                odd_status = '%s%s' % (wc_status_kind_map[ entry.repos_text_status ],
+                    wc_status_kind_map[ entry.repos_prop_status ])
             else:
                 odd_status = '  '
 
             lock_state = ' '
-            if file.entry is not None and hasattr( file.entry, 'lock_token' ):
-                if file.entry.lock_token is not None:
+            if entry.entry is not None and hasattr( entry.entry, 'lock_token' ):
+                if entry.entry.lock_token is not None:
                     lock_state = 'K'
 
-            if hasattr( file, 'repos_lock' ) and file.repos_lock is not None:
+            if hasattr( entry, 'repos_lock' ) and entry.repos_lock is not None:
                 lock_state = 'O'
 
-            if file.entry is not None and detailed:
+            if entry.entry is not None and detailed:
                 print( '%s%s %s %6d %6d %-14s %s' %
                     (state,
                     lock_state,
                     odd_status,
-                    file.entry.revision.number,
-                    file.entry.commit_revision.number,
-                    file.entry.commit_author,
-                    file.path) )
+                    entry.entry.revision.number,
+                    entry.entry.commit_revision.number,
+                    entry.entry.commit_author,
+                    entry.path) )
 
             elif detailed:
                 print( '%s%s %s %6s %6s %-14s %s' %
@@ -993,24 +1106,121 @@ class SvnCommand:
                     '',
                     '',
                     '',
-                    file.path) )
+                    entry.path) )
 
             elif update:
                 print( '%s%s %s %s' %
                     (state,
                     lock_state,
                     odd_status,
-                    file.path) )
+                    entry.path) )
 
             else:
-                if( file.text_status != pysvn.wc_status_kind.normal
-                or file.prop_status != pysvn.wc_status_kind.normal
+                if( entry.text_status != pysvn.wc_status_kind.normal
+                or entry.prop_status != pysvn.wc_status_kind.normal
                 or lock_state.strip() != ''):
-                    print( '%s%s %s' % (state, lock_state, file.path) )
+                    print( '%s%s %s' % (state, lock_state, entry.path) )
 
     cmd_st = cmd_status
     cmd_stat = cmd_status
 
+    def cmd_status2( self, args ):
+        recurse = args.getBooleanOption( '--non-recursive', False )
+        verbose = args.getBooleanOption( '--verbose', True )
+        quiet = args.getBooleanOption( '--quiet', True )
+        ignore = args.getBooleanOption( '--no-ignore', False )
+        update = args.getBooleanOption( '--show-updates', True )
+
+        positional_args = args.getPositionalArgs( 0 )
+        if len(positional_args) == 0:
+            all_entries = self.client.status2( '', recurse=recurse, get_all=verbose, ignore=ignore, update=update )
+            self._cmd_status2_print( all_entries, verbose, update, ignore, quiet )
+        else:
+            for arg in positional_args:
+                all_entries = self.client.status2( arg, recurse=recurse, get_all=verbose, ignore=ignore, update=update )
+                self._cmd_status2_print( all_entries, verbose, update, ignore, quiet )
+
+    def _cmd_status2_print( self, all_entries, detailed, update, ignore, quiet ):
+        all_entries.sort( key=self.key_by_path )
+        for entry in all_entries:
+            if entry.text_status == pysvn.wc_status_kind.ignored and ignore:
+                continue
+
+            if entry.text_status == pysvn.wc_status_kind.unversioned and quiet:
+                continue
+
+            if entry.text_status == pysvn.wc_status_kind.none and quiet:
+                continue
+
+            if entry.text_status == pysvn.wc_status_kind.none:
+                text_status = '?'
+            else:
+                text_status = wc_status_kind_map[ entry.text_status ]
+
+            state = '%s%s%s%s%s' % (text_status,
+                    wc_status_kind_map[ entry.prop_status ],
+                    ' L'[ entry.wc_is_locked ],
+                    ' +'[ entry.is_copied ],
+                    ' S'[ entry.is_switched ])
+
+            if( entry.repos_text_status != pysvn.wc_status_kind.none
+            or  entry.repos_prop_status != pysvn.wc_status_kind.none ):
+                odd_status = '%s%s' % (wc_status_kind_map[ entry.repos_text_status ],
+                    wc_status_kind_map[ entry.repos_prop_status ])
+            else:
+                odd_status = '  '
+
+            lock_state = ' '
+            if entry.lock is not None and entry.lock.token is not None:
+                lock_state = 'K'
+
+            if entry.repos_lock is not None:
+                lock_state = 'O'
+
+            # convert from abs to rel path
+            cwd = os.getcwd()
+            if entry.path == cwd:
+                path = '.'
+
+            elif entry.path.startswith( cwd + '/' ):
+                path = entry.path[len(cwd)+1:]
+
+            else:
+                path = entry.path
+
+            if detailed and entry.is_versioned:
+                print( '%s%s %s %6d %6d %-14s %s' %
+                    (state,
+                    lock_state,
+                    odd_status,
+                    entry.revision.number,
+                    entry.changed_revision.number,
+                    entry.changed_author,
+                    path) )
+
+            elif detailed:
+                print( '%s%s %s %6s %6s %-14s %s' %
+                    (state,
+                    lock_state,
+                    odd_status,
+                    '',
+                    '',
+                    '',
+                    path) )
+
+            elif update:
+                print( '%s%s %s %s' %
+                    (state,
+                    lock_state,
+                    odd_status,
+                    path) )
+
+            else:
+                if( entry.text_status != pysvn.wc_status_kind.normal
+                or entry.prop_status != pysvn.wc_status_kind.normal
+                or lock_state.strip() != ''):
+                    print( '%s%s %s' % (state, lock_state, path) )
+
     def cmd_switch( self, args ):
         recurse = args.getBooleanOption( '--non-recursive', False )
         revision = args.getOptionalRevision( '--revision', 'head' )
@@ -1051,6 +1261,26 @@ class SvnCommand:
 
     cmd_up = cmd_update
 
+    def cmd_vacuum( self, args ):
+        remove_unversioned_items = args.getBooleanOption( '--remove-unversioned-items', True )
+        remove_ignored_items = args.getBooleanOption( '--remove-ignored-items', True )
+        fix_recorded_timestamps = args.getBooleanOption( '--fix-recorded-timestamps', True )
+        vacuum_pristines = args.getBooleanOption( '--vacuum-pristines', True )
+        include_externals = args.getBooleanOption( '--include-externals', True )
+
+        positional_args = args.getPositionalArgs( 0, 1 )
+        if len(positional_args) == 0:
+            positional_args.append( '.' )
+
+        self.client.vacuum(
+                positional_args[0],
+                remove_unversioned_items=remove_unversioned_items,
+                remove_ignored_items=remove_ignored_items,
+                fix_recorded_timestamps=fix_recorded_timestamps,
+                vacuum_pristines=vacuum_pristines,
+                include_externals=include_externals
+                )
+
     def cmd_help( self, args ):
         print( 'Version: pysvn %d.%d.%d-%d' % pysvn.version,'svn %d.%d.%d-%s' % pysvn.svn_version )
         valid_cmd_names = [name for name in SvnCommand.__dict__.keys() if name.find('cmd_') == 0]
@@ -1067,18 +1297,32 @@ class SvnCommand:
 
 # key is long option name, value is 1 if need next arg as value
 long_opt_info = {
+    # svn_cmd.py control
     '--pause': 0,
+    '--pysvn-testing': 1,       # modify behaviour to assist testing pysvn
+    '--debug': 0,               # do debug stuff
 
+    # command options
     '--auto-props': 0,          # enable automatic properties
+    '--change-list': 1,         # changelist
     '--config-dir': 1,          # read user configuration files from directory ARG
     '--diff-cmd': 1,            # use ARG as diff command
     '--diff3-cmd': 1,           # use ARG as merge command
     '--dry-run': 0,             # try operation but make no changes
     '--editor-cmd': 1,          # use ARG as external editor
     '--encoding': 1,            # treat value as being in charset encoding ARG
+    '--extensions': 1,          # pass ARG as bundled options to GNU diff
+    '--fetch-locks': 0,         # as list to fetch lock info
+    '--file': 1,                # read data from file ARG
+    '--fix-recorded-timestamps': 0,
     '--force': 0,               # force operation to run
     '--force-log': 0,           # force validity of log message source
+    '--include-externals': 0,
+    '--include-externals': 0,   # include external information in list() output
     '--incremental': 0,         # give output suitable for concatenation
+    '--limit': 1,               # number of logs to fetch
+    '--message': 1,             # specify commit message ARG
+    '--native-eol': 1,          # native eol ARG
     '--new': 1,                 # use ARG as the newer target
     '--no-auth-cache': 0,       # do not cache authentication tokens
     '--no-auto-props': 0,       # disable automatic properties
@@ -1086,28 +1330,27 @@ long_opt_info = {
     '--no-ignore': 0,           # disregard default and svn:ignore property ignores
     '--no-remove-tempfiles': 0, # do not remove temp files
     '--non-interactive': 0,     # do no interactive prompting
+    '--non-recursive': 0,       # operate on single directory only
     '--notice-ancestry': 0,     # notice ancestry when calculating differences
     '--old': 1,                 # use ARG as the older target
     '--password': 1,            # specify a password ARG
+    '--quiet': 0,               # print as little as possible
+    '--recursive': 0,           # descend recursively
     '--relocate': 0,            # relocate via URL-rewriting
+    '--remove-ignored-items': 0,
+    '--remove-unversioned-items': 0,
+    '--revision': 1,            # revision X or X:Y range.  X or Y can be one of:
     '--revprop': 0,             # operate on a revision property (use with -r)
+    '--show-inherited-props': 0, # show inherited props
+    '--show-updates': 0,        # display update information
+    '--skip-checks': 0,         # skip-checks
     '--strict': 0,              # use strict semantics
     '--targets': 1,             # pass contents of file ARG as additional args
     '--username': 1,            # specify a username ARG
+    '--vacuum-pristines': 0,
+    '--verbose': 0,             # print extra information
     '--version': 0,             # print client version info
     '--xml': 0,                 # output in xml
-    '--file': 1,                # read data from file ARG
-    '--native-eol': 1,          # native eol ARG
-    '--non-recursive': 0,       # operate on single directory only
-    '--recursive': 0,           # descend recursively
-    '--message': 1,             # specify commit message ARG
-    '--quiet': 0,               # print as little as possible
-    '--revision': 1,            # revision X or X:Y range.  X or Y can be one of:
-    '--show-updates': 0,        # display update information
-    '--verbose': 0,             # print extra information
-    '--extensions': 1,          # pass ARG as bundled options to GNU diff
-    '--pysvn-testing': 1,       # modify behaviour to assist testing pysvn
-    '--debug': 0,               # do debug stuff
 }
 
 # map short name to long
