@@ -1,6 +1,6 @@
 //
 // ====================================================================
-// Copyright (c) 2003-2013 Barry A Scott.  All rights reserved.
+// Copyright (c) 2003-2016 Barry A Scott.  All rights reserved.
 //
 // This software is licensed as described in the file LICENSE.txt,
 // which you should have received as part of this distribution.
@@ -8,6 +8,8 @@
 // ====================================================================
 //
 #include "Python.h"
+#include "CXX/Version.hxx"
+
 #include "CXX/Objects.hxx"
 #include "CXX/Extensions.hxx"
 #include <iostream>
@@ -224,6 +226,9 @@ public:
     Py::Object cmd_add_to_changelist( const Py::Tuple& args, const Py::Dict &kws );
 #endif
     Py::Object cmd_annotate( const Py::Tuple& args, const Py::Dict &kws );
+#if defined( PYSVN_HAS_CLIENT_ANNOTATE5 )
+    Py::Object cmd_annotate2( const Py::Tuple& args, const Py::Dict &kws );
+#endif
     Py::Object cmd_cat( const Py::Tuple& args, const Py::Dict &kws );
     Py::Object cmd_checkout( const Py::Tuple& args, const Py::Dict &kws );
     Py::Object cmd_cleanup( const Py::Tuple& args, const Py::Dict &kws );
@@ -270,11 +275,22 @@ public:
 #endif
 #if defined( PYSVN_HAS_CLIENT_PATCH )
     Py::Object cmd_patch( const Py::Tuple& args, const Py::Dict &kws );
+#endif
+#if defined( PYSVN_HAS_CLIENT_PROPSET_LOCAL )
+    Py::Object cmd_propdel_local( const Py::Tuple& args, const Py::Dict &kws );
+    Py::Object cmd_propset_local( const Py::Tuple& args, const Py::Dict &kws );
+    Py::Object common_propset_local( FunctionArguments &args, bool is_set );
+#endif
+#if defined( PYSVN_HAS_CLIENT_PROPSET_REMOTE )
+    Py::Object cmd_propdel_remote( const Py::Tuple& args, const Py::Dict &kws );
+    Py::Object cmd_propset_remote( const Py::Tuple& args, const Py::Dict &kws );
+    Py::Object common_propset_remote( FunctionArguments &args, bool is_set );
 #endif
     Py::Object cmd_propdel( const Py::Tuple& args, const Py::Dict &kws );
+    Py::Object cmd_propset( const Py::Tuple& args, const Py::Dict &kws );
+    Py::Object common_propset( FunctionArguments &args, bool is_set );
     Py::Object cmd_propget( const Py::Tuple& args, const Py::Dict &kws );
     Py::Object cmd_proplist( const Py::Tuple& args, const Py::Dict &kws );
-    Py::Object cmd_propset( const Py::Tuple& args, const Py::Dict &kws );
     Py::Object cmd_relocate( const Py::Tuple& args, const Py::Dict &kws );
     Py::Object cmd_remove( const Py::Tuple& args, const Py::Dict &kws );
 #ifdef PYSVN_HAS_CLIENT_REMOVE_FROM_CHANGELISTS
@@ -283,9 +299,13 @@ public:
     Py::Object cmd_resolved( const Py::Tuple& args, const Py::Dict &kws );
     Py::Object cmd_revert( const Py::Tuple& args, const Py::Dict &kws );
     Py::Object cmd_revpropdel( const Py::Tuple& args, const Py::Dict &kws );
+    Py::Object cmd_revpropset( const Py::Tuple& args, const Py::Dict &kws );
+    Py::Object common_revpropset( FunctionArguments &args, bool is_set );
     Py::Object cmd_revpropget( const Py::Tuple& args, const Py::Dict &kws );
     Py::Object cmd_revproplist( const Py::Tuple& args, const Py::Dict &kws );
-    Py::Object cmd_revpropset( const Py::Tuple& args, const Py::Dict &kws );
+#if defined( PYSVN_HAS_CLIENT_STATUS5 )
+    Py::Object cmd_status2( const Py::Tuple& args, const Py::Dict &kws );
+#endif
     Py::Object cmd_status( const Py::Tuple& args, const Py::Dict &kws );
     Py::Object cmd_switch( const Py::Tuple& args, const Py::Dict &kws );
 #if defined( PYSVN_HAS_CLIENT_LOCK )
@@ -295,7 +315,9 @@ public:
     Py::Object cmd_upgrade( const Py::Tuple& args, const Py::Dict &kws );
 #endif
     Py::Object cmd_update( const Py::Tuple& args, const Py::Dict &kws );
-
+#if defined( PYSVN_HAS_CLIENT_VACUUM )
+    Py::Object cmd_vacuum( const Py::Tuple& args, const Py::Dict &kws );
+#endif
     // SVN commands
     Py::Object is_url( const Py::Tuple& args, const Py::Dict &kws );
     Py::Object get_auth_cache( const Py::Tuple& args, const Py::Dict &kws );
@@ -339,6 +361,9 @@ private:
     int                 m_exception_style;
     int                 m_commit_info_style;
 
+#if defined( PYSVN_HAS_CLIENT_STATUS_T )
+    DictWrapper         m_wrapper_status2;
+#endif
     DictWrapper         m_wrapper_status;
     DictWrapper         m_wrapper_entry;
     DictWrapper         m_wrapper_info;
@@ -349,6 +374,7 @@ private:
     DictWrapper         m_wrapper_dirent;
     DictWrapper         m_wrapper_wc_info;
     DictWrapper         m_wrapper_diff_summary;
+    DictWrapper         m_wrapper_commit_info;
 };
 
 class pysvn_transaction : public Py::PythonExtension<pysvn_transaction>
@@ -432,6 +458,7 @@ public:
     void check();
 
     bool hasArg( const char *arg_name );
+    bool hasArgNotNone( const char *arg_name );
     Py::Object getArg( const char *arg_name );
 
     bool getBoolean( const char *name );
@@ -458,8 +485,9 @@ public:
     svn_wc_conflict_choice_t getWcConflictChoice( const char *choice_name );
 #endif
 
-private:
+public:
     const std::string           m_function_name;
+private:
     const argument_description  *m_arg_desc;
     const Py::Tuple             &m_args;
     const Py::Dict              &m_kws;
@@ -469,6 +497,34 @@ private:
     Py::Tuple::size_type        m_max_args;
 };
 
+//------------------------------------------------------------
+#if defined( PYSVN_HAS_COMMIT_CALLBACK2_T )
+extern "C" svn_error_t *CommitInfoResult_callback( const svn_commit_info_t *commit_info, void *baton, apr_pool_t *pool );
+
+class CommitInfoResult
+{
+    friend svn_error_t *CommitInfoResult_callback( const svn_commit_info_t *commit_info, void *baton, apr_pool_t *pool );
+
+public:
+    CommitInfoResult( SvnPool &pool );
+    ~CommitInfoResult();
+
+    // needed to call svn API
+    svn_commit_callback2_t callback() { return &CommitInfoResult_callback; }
+    void *baton() { return static_cast< void * >( this ); }
+
+    // convert to python
+    int count();
+    const svn_commit_info_t *result( int index );
+
+private:
+    static CommitInfoResult *castBaton( void *baton_ ) { return static_cast<CommitInfoResult *>( baton_ ); }
+
+    apr_array_header_t  *m_all_results;
+    SvnPool             &m_pool;
+};
+#endif
+
 //------------------------------------------------------------
 template<TEMPLATE_TYPENAME T>
 class EnumString
@@ -742,6 +798,7 @@ typedef signed long long signed_int64;
 #endif
 
 
+
 //--------------------------------------------------------------------------------
 //
 //    PythonAllowThreads provides a exception safe
@@ -792,11 +849,17 @@ private:
 
 //--------------------------------------------------------------------------------
 extern Py::Object utf8_string_or_none( const char *str );
-extern Py::Object path_string_or_none( const char *str, SvnPool &pool );
 extern Py::Object utf8_string_or_none( const std::string &str );
+
+extern Py::Object path_string_or_none( const char *str, SvnPool &pool );
+extern Py::Object path_string_or_none( const std::string &str, SvnPool &pool );
+
 extern apr_time_t convertStringToTime( const std::string &text, apr_time_t now, SvnPool &pool );
 
 extern Py::Object propsToObject( apr_hash_t *props, SvnPool &pool );
+#if defined( PYSVN_HAS_CLIENT_PROPGET5 )
+extern Py::Object inheritedPropsToObject( apr_array_header_t *inherited_props, SvnPool &pool );
+#endif
 extern Py::Object revnumListToObject( apr_array_header_t *revs, SvnPool &pool );
 extern void proplistToObject( Py::List &py_path_propmap_list, apr_array_header_t *props, SvnPool &pool );
 extern Py::Bytes asUtf8Bytes( Py::Object obj );
@@ -809,6 +872,21 @@ extern Py::Object direntsToObject( apr_hash_t *props, SvnPool &pool );
 Py::Object toObject( apr_time_t t );
 Py::Object toObject( pysvn_commit_info_t *commit_info, int commit_style );
 
+#if defined( PYSVN_HAS_SVN_COMMIT_INFO_T )
+Py::Object toObject( CommitInfoResult &commit_info, const DictWrapper &wrapper_commit_info, int commit_style );
+#endif
+
+#if defined( PYSVN_HAS_CLIENT_STATUS_T )
+extern Py::Object toObject
+    (
+    Py::String path,
+    svn_client_status_t &svn_status,
+    SvnPool &pool,
+    const DictWrapper &wrapper_status2,
+    const DictWrapper &wrapper_lock
+    );
+#endif
+
 extern Py::Object toObject
     (
     Py::String path,
@@ -818,12 +896,14 @@ extern Py::Object toObject
     const DictWrapper &wrapper_entry,
     const DictWrapper &wrapper_lock
     );
+
 extern Py::Object toObject
     (
     const svn_wc_entry_t &svn_entry,
     SvnPool &pool,
     const DictWrapper &wrapper_entry
     );
+
 #if defined( PYSVN_HAS_CLIENT_INFO )
 extern Py::Object toObject
     (
@@ -833,6 +913,16 @@ extern Py::Object toObject
     const DictWrapper &wrapper_wc_info
     );
 #endif
+#if defined( PYSVN_HAS_CLIENT_INFO3 )
+extern Py::Object toObject
+    (
+    const svn_client_info2_t &info,
+    SvnPool &pool,
+    const DictWrapper &wrapper_info,
+    const DictWrapper &wrapper_lock,
+    const DictWrapper &wrapper_wc_info
+    );
+#endif
 #if defined( PYSVN_HAS_CLIENT_LOCK )
 extern Py::Object toObject
     (
