@@ -1,6 +1,6 @@
 #
 # ====================================================================
-# Copyright (c) 2005-2011 Barry A Scott.  All rights reserved.
+# Copyright (c) 2005-2016 Barry A Scott.  All rights reserved.
 #
 # This software is licensed as described in the file LICENSE.txt,
 # which you should have received as part of this distribution.
@@ -25,7 +25,7 @@ class SetupError(Exception):
     pass
 
 # version of PyCXX that we require
-pycxx_version = (6, 2, 6)
+min_pycxx_version = (6, 2, 8)
 
 _debug = False
 
@@ -54,6 +54,7 @@ def cmd_help( argv ):
 class Options:
     all_options_info = {
         '--arch':               (2, '<arch>'),
+        '--distro-dir':         (2, '<dir>'),
         '--apr-inc-dir':        (1, '<dir>'),
         '--apu-inc-dir':        (1, '<dir>'),
         '--apr-lib-dir':        (1, '<dir>'),
@@ -68,9 +69,9 @@ class Options:
         '--svn-inc-dir':        (1, '<dir>'),
         '--svn-lib-dir':        (1, '<dir>'),
         '--svn-bin-dir':        (1, '<dir>'),
-        '--svn-root-dir':       (1, '<dir>'),
         '--verbose':            (0, None),
         '--disable-deprecated-functions-warnings': (0, None),
+        '--link-python-framework-via-dynamic-lookup': (0, None),
         }
 
     def __init__( self, argv ):
@@ -165,7 +166,7 @@ class Setup:
                 ,sys.exec_prefix) )
 
         if self.options.hasOption( '--platform' ):
-            self.platform = self.options.getOption( '--platform' )
+            self.platform = self.options.getOption( '--platform' ).lower()
 
         else:
             if sys.platform == 'darwin' and os.path.exists( '/System/Library/CoreServices/SystemVersion.plist' ):
@@ -189,7 +190,7 @@ class Setup:
             else:
                 raise SetupError( 'Cannot automatically detect your platform use --platform option' )
 
-        if self.platform == 'win32':
+        if self.platform in ('win32', 'win64'):
             self.c_utils = Win32CompilerMSVC90( self )
             self.c_pysvn = Win32CompilerMSVC90( self )
 
@@ -201,6 +202,14 @@ class Setup:
             self.c_utils = LinuxCompilerGCC( self )
             self.c_pysvn = LinuxCompilerGCC( self )
 
+        elif self.platform == 'kfreebsd':
+            self.c_utils = LinuxCompilerGCC( self )
+            self.c_pysvn = LinuxCompilerGCC( self )
+
+        elif self.platform == 'hurd':
+            self.c_utils = LinuxCompilerGCC( self )
+            self.c_pysvn = LinuxCompilerGCC( self )
+
         elif self.platform == 'freebsd':
             self.c_utils = FreeBsdCompilerGCC( self )
             self.c_pysvn = FreeBsdCompilerGCC( self )
@@ -232,6 +241,11 @@ class Setup:
             Source( self.c_pysvn, '%(PYCXX_SRC)s/IndirectPythonInterface.cxx' ),
             ]
 
+        # after 7.0.0 need to compile new file
+        if self.c_pysvn.pycxx_version >= (7, 0, 0):
+            self.pycxx_obj_file.append(
+                Source( self.c_pysvn, '%(PYCXX_SRC)s/cxx_exceptions.cxx' ) )
+
         pysvn_headers = ['pysvn.hpp', 'pysvn_docs.hpp', 'pysvn_svnenv.hpp', 'pysvn_static_strings.hpp', 'pysvn_version.hpp']
         self.pysvn_obj_files = [
             Source( self.c_pysvn, 'pysvn.cpp', pysvn_headers ),
@@ -282,7 +296,7 @@ class Setup:
 
     def generateSourcesMakefile( self ):
         if not self.options.parseOptions():
-            raise SetupError( 'failed to parse options' )
+            raise SetupError( 'Failed to parse options' )
 
         self.__makefile = open( 'Makefile', 'wt' )
 
@@ -304,7 +318,7 @@ class Setup:
         self.__makefile = None
 
         if not self.options.checkAllOptionsUsed():
-            raise SetupError( 'not all options used' )
+            raise SetupError( 'Not all options used' )
 
         return 0
 
@@ -328,6 +342,8 @@ class Setup:
             TestCase( self.c_pysvn, '06' ),
             TestCase( self.c_pysvn, '07' ),
             TestCase( self.c_pysvn, '08', (1,7,0) ),
+            TestCase( self.c_pysvn, '09', (1,7,0) ),
+            TestCase( self.c_pysvn, '10', (1,9,0) ),
             ] )
 
         all_extra_test_cases = self.__filterTestCases( [
@@ -364,8 +380,7 @@ class Compiler:
 
         self.__variables = {}
 
-        self._addVar( 'PYCXX_VER',       '6.2.4' )
-        self._addVar( 'DEBUG',           'NDEBUG')    
+        self._addVar( 'DEBUG',           'NDEBUG')
 
         self._addVar( 'PYSVN_SRC_DIR', os.path.dirname( os.getcwd() ) )
 
@@ -410,7 +425,17 @@ class Compiler:
 
         self._addVar( 'PYTHON',         sys.executable )
 
-        self._addVar( 'PYCXX',          self.find_pycxx() )
+        pycxx_dir = self.find_pycxx()
+        self._addVar( 'PYCXX',          pycxx_dir )
+
+        self.pycxx_version = self.__getPyCxxVersion( pycxx_dir )
+
+        # assume that newer version are always usable
+        if self.pycxx_version < min_pycxx_version:
+            raise SetupError( 'PyCXX version %d.%d.%d required, but found %d.%d.%d.' %
+                                (pycxx_version[0], pycxx_version[1], pycxx_version[2]
+                                ,self.pycxx_version[0], self.pycxx_version[1], self.pycxx_version[2]) )
+
         self._addVar( 'PYCXX_SRC',      self.find_pycxx_src() )
         svn_inc = self.find_svn_inc()
         self._addVar( 'SVN_INC',        svn_inc )
@@ -465,30 +490,34 @@ class Compiler:
 
 
     def find_pycxx( self ):
-        pycxx_dir = self.find_dir(
+        return self.find_dir(
                     'PyCXX include',
                     '--pycxx-dir',
                     None,
                     self._find_paths_pycxx_dir,
                     'CXX/Version.hxx' )
 
-        major_match = False
-        minor_match = False
+    def __getPyCxxVersion( self, pycxx_dir ):
+        major = None
+        minor = None
+        patch = None
         f = open( os.path.join( pycxx_dir, 'CXX/Version.hxx' ) )
         for line in f:
             words = line.split()
-            if 'PYCXX_VERSION_MAJOR' in words:
-                major_match = int( words[2] ) == pycxx_version[0]
-            if 'PYCXX_VERSION_MINOR' in words:
-                minor_match = int( words[2] ) == pycxx_version[1]
+            if len(words) < 3:
+                continue
 
-        if not major_match and not minor_match:
-            raise SetupError( 'PyCXX version not as required.' )
+            if 'PYCXX_VERSION_MAJOR' == words[1]:
+                major = int( words[2] )
+            if 'PYCXX_VERSION_MINOR' == words[1]:
+                minor = int( words[2] )
+            if 'PYCXX_VERSION_PATCH' == words[1]:
+                patch = int( words[2] )
 
-        return pycxx_dir
+        return (major, minor, patch)
 
     def find_pycxx_src( self ):
-        v = {'PYCXX': self.find_pycxx()}
+        v = {'PYCXX': self.expand( '%(PYCXX)s' )}
 
         return self.find_dir(
                     'PyCXX Source',
@@ -520,12 +549,23 @@ class Compiler:
                     'svnadmin%s' % (self.getProgramExt(),) )
 
     def find_svn_lib( self ):
-        folder = self.find_dir(
-                    'SVN library',
-                    '--svn-lib-dir',
-                    'lib',
-                    self._find_paths_svn_lib,
-                    self.get_lib_name_for_platform( 'libsvn_client-1' ) )
+        last_exception = None
+        for lib_name in self.get_lib_name_for_platform( 'libsvn_client-1' ):
+            try:
+                folder = self.find_dir(
+                        'SVN library',
+                        '--svn-lib-dir',
+                        'lib',
+                        self._find_paths_svn_lib,
+                        lib_name )
+                break
+
+            except SetupError as e:
+                last_exception = e
+
+        if last_exception is not None:
+            raise last_exception
+
         # if we are using the Fink SVN then remember this
         self.is_mac_os_x_fink = folder.startswith( '/sw/' )
         self.is_mac_os_x_darwin_ports = folder.startswith( '/opt/local/' )
@@ -535,7 +575,7 @@ class Compiler:
         return self.find_dir(
             'APR include',
             '--apr-inc-dir',
-            None,
+            'include',
             self._find_paths_apr_inc,
             'apr.h' )
 
@@ -543,40 +583,42 @@ class Compiler:
         return self.find_dir(
             'APR include',
             '--apu-inc-dir',
-            None,
+            'include',
             self._find_paths_apr_util_inc,
             'apu.h' )
 
     def find_apr_lib( self ):
         last_exception = None
-        lib_list = [(self.get_lib_name_for_platform( 'libapr-1' ), 'apr-1'),
-                    (self.get_lib_name_for_platform( 'libapr-0' ), 'apr-0')]
+        all_lib_names = self.get_lib_name_for_platform( 'libapr-1' )
 
-        for apr_libname, self.lib_apr in lib_list:
+        for lib_name in all_lib_names:
             try:
                 return self.find_dir(
-                    'APR library',
-                    '--apr-lib-dir',
-                    'lib',
-                    [],
-                    apr_libname )
+                        'APR library',
+                        '--apr-lib-dir',
+                        'lib',
+                        [],
+                        lib_name )
+
             except SetupError:
                 pass
 
-        for apr_libname, self.lib_apr in lib_list:
+        for lib_name in all_lib_names:
             try:
                 return self.find_dir(
-                    'APR library',
-                    '--apr-lib-dir',
-                    None,
-                    self._find_paths_apr_lib,
-                    apr_libname )
+                        'APR library',
+                        '--apr-lib-dir',
+                        None,
+                        self._find_paths_apr_lib,
+                        lib_name )
+
             except SetupError as e:
                 last_exception = e
+
         raise last_exception
 
     def get_lib_name_for_platform( self, libname ):
-        raise NotImplementedError
+        raise NotImplementedError( 'get_lib_name_for_platform' )
 
     def find_dir( self, name, kw, svn_root_suffix, base_dir_list, check_file ):
         dirname = self.__find_dir( name, kw, svn_root_suffix, base_dir_list, check_file )
@@ -593,10 +635,6 @@ class Compiler:
         if self.options.hasOption( kw ):
             base_dir_list = [self.options.getOption( kw )]
 
-        elif( self.options.hasOption( '--svn-root-dir' )
-        and svn_root_suffix is not None ):
-            base_dir_list = ['%s/%s' % (self.options.getOption( '--svn-root-dir' ), svn_root_suffix)]
-
         debug( '__find_dir base_dir_list=%r' % (base_dir_list,) )
 
         # expect to find check_file in one of the dir
@@ -607,7 +645,7 @@ class Compiler:
             if os.path.exists( full_check_file ):
                 return os.path.abspath( dirname )
 
-        raise SetupError( 'cannot find %s %s - use %s' % (name, check_file, kw) )
+        raise SetupError( 'Cannot find %s %s - use %s' % (name, check_file, kw) )
 
     def __getSvnVersion( self, svn_include ):
         all_svn_version_lines = open( os.path.join( svn_include, 'svn_version.h' ) ).readlines()
@@ -639,18 +677,23 @@ class Win32CompilerMSVC90(Compiler):
         self._find_paths_pycxx_src = [
                         '%(PYCXX)s/Src',
                         ]
-        self._find_paths_svn_inc = []
-        self._find_paths_svn_bin = []
-        self._find_paths_svn_lib = []
-        self._find_paths_apr_inc = []
-        self._find_paths_apr_util_inc = []
-        self._find_paths_apr_lib = []
 
-        if self.options.hasOption( '--svn-root-dir' ):
-            svn_root_dir = self.options.getOption( '--svn-root-dir' )
-            self._find_paths_apr_inc.append( r'%s\include\apr' % (svn_root_dir,) )
-            self._find_paths_apr_util_inc.append( r'%s\include\apr-util' % (svn_root_dir,) )
-            self._find_paths_apr_lib.append( r'%s\lib\apr' % (svn_root_dir,) )
+        if self.options.hasOption( '--distro-dir' ):
+            all_distro_dirs = self.options.getOption( '--distro-dir' )
+            self._find_paths_apr_inc = [r'%s\include' % (distro_dir,) for distro_dir in all_distro_dirs]
+            self._find_paths_apr_util_inc = [r'%s\include' % (distro_dir,) for distro_dir in all_distro_dirs]
+            self._find_paths_apr_lib = [r'%s\lib' % (distro_dir,) for distro_dir in all_distro_dirs]
+            self._find_paths_svn_inc = [r'%s\include\subversion-1' % (distro_dir,) for distro_dir in all_distro_dirs]
+            self._find_paths_svn_lib = [r'%s\lib' % (distro_dir,) for distro_dir in all_distro_dirs]
+            self._find_paths_svn_bin = [r'%s\bin' % (distro_dir,) for distro_dir in all_distro_dirs]
+
+        else:
+            self._find_paths_svn_inc = []
+            self._find_paths_svn_bin = []
+            self._find_paths_svn_lib = []
+            self._find_paths_apr_inc = []
+            self._find_paths_apr_util_inc = []
+            self._find_paths_apr_lib = []
 
         self._addVar( 'PYTHON_DIR',     sys.exec_prefix )
         self._addVar( 'PYTHON_INC',     r'%(PYTHON_DIR)s\include' )
@@ -687,7 +730,7 @@ class Win32CompilerMSVC90(Compiler):
 
     def ruleLinkProgram( self, target ):
         pyd_filename = target.getTargetFilename()
-        pdf_filename = target.getTargetFilename( '.pdf' )
+        pdb_filename = target.getTargetFilename( '.pdb' )
 
         all_objects = [source.getTargetFilename() for source in target.all_sources]
 
@@ -697,13 +740,13 @@ class Win32CompilerMSVC90(Compiler):
         rules.append( '%s : %s' % (pyd_filename, ' '.join( all_objects )) )
         rules.append( '\t@echo Link %s' % (pyd_filename,) )
         rules.append( '\t@$(LDEXE)  %%(CCCFLAGS)s /Fe%s /Fd%s %s $(LDLIBS)' %
-                            (pyd_filename, pdf_filename, ' '.join( all_objects )) )
+                            (pyd_filename, pdb_filename, ' '.join( all_objects )) )
 
         self.makePrint( self.expand( '\n'.join( rules ) ) )
 
     def ruleLinkShared( self, target ):
         pyd_filename = target.getTargetFilename()
-        pdf_filename = target.getTargetFilename( '.pdf' )
+        pdb_filename = target.getTargetFilename( '.pdb' )
 
         all_objects = [source.getTargetFilename() for source in target.all_sources]
 
@@ -713,7 +756,7 @@ class Win32CompilerMSVC90(Compiler):
         rules.append( '%s : %s' % (pyd_filename, ' '.join( all_objects )) )
         rules.append( '\t@echo Link %s' % (pyd_filename,) )
         rules.append( '\t@$(LDSHARED)  %%(CCCFLAGS)s /Fe%s /Fd%s %s %%(PYTHON_LIB)s\python%d%d.lib $(LDLIBS)' %
-                            (pyd_filename, pdf_filename, ' '.join( all_objects ), sys.version_info[0], sys.version_info[1]) )
+                            (pyd_filename, pdb_filename, ' '.join( all_objects ), sys.version_info[0], sys.version_info[1]) )
 
         self.makePrint( self.expand( '\n'.join( rules ) ) )
 
@@ -740,7 +783,7 @@ class Win32CompilerMSVC90(Compiler):
         self.makePrint( self.expand( '\n'.join( rules ) ) )
 
     def get_lib_name_for_platform( self, libname ):
-        return '%s.lib' % (libname,)
+        return ['%s.lib' % (libname,)]
 
     def setupUtilities( self ):
         self._addVar( 'CCCFLAGS',
@@ -755,7 +798,6 @@ class Win32CompilerMSVC90(Compiler):
         self._pysvnModuleSetup()
         self._addVar( 'PYSVN_MODULE_BASENAME', self.pysvn_module_name )
 
-
         py_cflags_list = [
                                         r'/Zi /MT /EHsc ',
                                         r'/I. /I%(APR_INC)s /I%(APU_INC)s /I%(SVN_INC)s',
@@ -765,6 +807,10 @@ class Win32CompilerMSVC90(Compiler):
                                         r'/DWIN32',
                                         r'/D%(DEBUG)s',
                                         ]
+        if self.pycxx_version >= (7, 0, 0):
+            # PYSVN uses PYCXX in backward compat mode
+            py_cflags_list.append( r'/DPYCXX_6_2_COMPATIBILITY=1' )
+
         for define, value in self.py_module_defines:
             py_cflags_list.append( '/D%s=%s' % (define, value) )
 
@@ -788,17 +834,20 @@ class Win32CompilerMSVC90(Compiler):
                     r'%(SVN_LIB)s\libsvn_repos-1.lib',
                     r'%(SVN_LIB)s\libsvn_subr-1.lib',
                     r'%(SVN_LIB)s\libsvn_wc-1.lib',
-                    r'%(SVN_LIB)s\apr-iconv\libapriconv-1.lib',
-                    r'%(SVN_LIB)s\apr-util\libaprutil-1.lib',
-                    r'%(SVN_LIB)s\apr-util\xml.lib',
-                    r'%(SVN_LIB)s\apr\libapr-1.lib',
+                    r'%(APR_LIB)s\libapriconv-1.lib',
+                    r'%(APR_LIB)s\libaprutil-1.lib',
+                    #r'%(APR_LIB)s\xml.lib',
+                    r'%(APR_LIB)s\libapr-1.lib',
                     ]
-        if os.path.exists( self.expand( r'%(SVN_LIB)s\serf\serf-1.lib' ) ):
-            ldlibs.append( r'%(SVN_LIB)s\serf\serf-1.lib' )
-        elif os.path.exists( self.expand( r'%(SVN_LIB)s\libsvn_ra_neon-1.lib' ) ):
-            ldlibs.append( r'%(SVN_LIB)s\libsvn_ra_neon-1.lib' )
-        else:
-            raise SetupError( 'Cannot find serf or neon' )
+        if False:
+            if os.path.exists( self.expand( r'%(SVN_LIB)s\serf\serf-1.lib' ) ):
+                ldlibs.append( r'%(SVN_LIB)s\serf\serf-1.lib' )
+
+            elif os.path.exists( self.expand( r'%(SVN_LIB)s\libsvn_ra_neon-1.lib' ) ):
+                ldlibs.append( r'%(SVN_LIB)s\libsvn_ra_neon-1.lib' )
+
+            else:
+                raise SetupError( 'Cannot find serf or neon' )
 
         ldlibs.append( r'ws2_32.lib' )
 
@@ -840,7 +889,7 @@ class Win32CompilerMSVC90(Compiler):
         rules.append( '\t' 'wb-diff.cmd test-%(TN)s.win32.known_good-%(KGV)s.log.clean test-%(TN)s.win32.new.log.clean' % v )
         rules.append( '' )
         rules.append( 'new-%(TN)s: test-%(TN)s.win32.new.log' % v )
-        rules.append( '\t' 'copy test-%(TN)s.win32.new.log test-%(TN)s.win32.known_good-%(KGV)s.log' % v )
+        rules.append( '\t' 'move /y test-%(TN)s.win32.new.log test-%(TN)s.win32.known_good-%(KGV)s.log' % v )
 
         self.makePrint( self.expand( '\n'.join( rules ) ) )
 
@@ -913,8 +962,8 @@ class CompilerGCC(Compiler):
         rules = []
 
         rules.append( '%s: %s %s' % (obj_filename, target.src_filename, ' '.join( target.all_dependencies )) )
-        rules.append( '\t@echo Compile: %s into %s' % (target.src_filename, target) )
-        rules.append( '\t@%%(CC)s -c %%(CCCFLAGS)s -o%s  %s' % (obj_filename, target.src_filename) )
+        rules.append( '\t@echo Compile: %s into %s' % (target.src_filename, obj_filename) )
+        rules.append( '\t@%%(CC)s -c %%(CCFLAGS)s -o%s  %s' % (obj_filename, target.src_filename) )
 
         self.makePrint( self.expand( '\n'.join( rules ) ) )
 
@@ -971,6 +1020,16 @@ class MacOsxCompilerGCC(CompilerGCC):
 
         print( 'Info: MACOSX_DEPLOYMENT_TARGET is %s' % (self.macosx_deployment_target,) )
 
+        xcode_path = '/Applications/Xcode.app'
+        xcode_version_path = os.path.join( xcode_path, 'Contents', 'version.plist' )
+        if os.path.exists( xcode_version_path ):
+            import plistlib
+            xcode_version_info = plistlib.readPlist( '/Applications/Xcode.app/Contents/version.plist')
+            self.xcode_version = [int(x) for x in xcode_version_info['CFBundleShortVersionString'].split( '.' )]
+
+        else:
+            self.xcode_version = None
+
         if self.options.hasOption( '--arch' ):
             arch_options = ' '.join( ['-arch %s' % (arch,) for arch in self.options.getOption( '--arch' )] )
         else:
@@ -980,55 +1039,66 @@ class MacOsxCompilerGCC(CompilerGCC):
         self._addVar( 'CC',             'gcc %s' % (arch_options,) )
 
         self._find_paths_pycxx_dir = [
-                        '../Import/pycxx-%d.%d.%d' % pycxx_version,
+                        '../Import/pycxx-%d.%d.%d' % min_pycxx_version,
                         distutils.sysconfig.get_python_inc() # typical Linux
                         ]
-        
+
         self._find_paths_pycxx_src = [
                         '%(PYCXX)s/Src',
                         '/usr/share/python%s/CXX' % (distutils.sysconfig.get_python_version(),) # typical Linux
                         ]
-        self._find_paths_svn_inc = [
-                        '/opt/local/include/subversion-1',      # Darwin - darwin ports
-                        '/sw/include/subversion-1',             # Darwin - Fink
-                        '/usr/include/subversion-1',            # typical Linux
-                        '/usr/local/include/subversion-1',      # typical *BSD
-                        ]
-        self._find_paths_svn_bin = [
-                        '/opt/local/bin',                        # Darwin - darwin ports
-                        '/sw/bin',                               # Darwin - Fink
-                        '/usr/bin',                              # typical Linux
-                        '/usr/local/bin',                        # typical *BSD
-                        ]
-        self._find_paths_svn_lib = [
-                        '/opt/local/lib',                       # Darwin - darwin ports
-                        '/sw/lib',                              # Darwin - Fink
-                        '/usr/lib',                             # typical Linux
-                        '/usr/local/lib',                       # typical *BSD
-                        ]
-        self._find_paths_apr_inc = [
-                        '/opt/local/include/apr-1',             # Darwin - darwin ports
-                        '/sw/include/apr-1',                    # Darwin - fink
-                        '/usr/include/apr-1',                   # typical Linux
-                        '/usr/local/apr/include/apr-1',         # Mac OS X www.metissian.com
-                        '/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX%s.sdk/usr/include/apr-1' % (self.macosx_deployment_target,),
-                        ]
-        self._find_paths_apr_util_inc = self._find_paths_apr_inc
-        self._find_paths_apr_lib = [
-                        '/opt/local/lib',                       # Darwin - darwin ports
-                        '/sw/lib',                              # Darwin - fink
-                        '/usr/lib64',                           # typical 64bit Linux
-                        '/usr/lib',                             # typical Linux
-                        '/usr/local/lib64',                     # typical 64bit Linux
-                        '/usr/local/lib',                       # typical *BSD
-                        '/usr/local/apr/lib',                   # Mac OS X www.metissian.com
-                        '/usr/pkg/lib',                         # netbsd
-                        ]
+
+        if self.options.hasOption( '--distro-dir' ):
+            all_distro_dirs = self.options.getOption( '--distro-dir' )
+            self._find_paths_svn_inc = ['%s/include/subversion-1' % (distro_dir,) for distro_dir in all_distro_dirs]
+            self._find_paths_svn_bin = ['%s/bin' % (distro_dir,) for distro_dir in all_distro_dirs]
+            self._find_paths_svn_lib = ['%s/lib' % (distro_dir,) for distro_dir in all_distro_dirs]
+            self._find_paths_apr_inc = ['%s/include/apr-1' % (distro_dir,) for distro_dir in all_distro_dirs]
+            self._find_paths_apr_util_inc = self._find_paths_apr_inc
+            self._find_paths_apr_lib = ['%s/lib' % (distro_dir,) for distro_dir in all_distro_dirs]
+
+        else:
+            self._find_paths_svn_inc = [
+                            '/opt/local/include/subversion-1',      # Darwin - darwin ports
+                            '/sw/include/subversion-1',             # Darwin - Fink
+                            '/usr/include/subversion-1',            # typical Linux
+                            '/usr/local/include/subversion-1',      # typical *BSD
+                            ]
+            self._find_paths_svn_bin = [
+                            '/opt/local/bin',                        # Darwin - darwin ports
+                            '/sw/bin',                               # Darwin - Fink
+                            '/usr/bin',                              # typical Linux
+                            '/usr/local/bin',                        # typical *BSD
+                            ]
+            self._find_paths_svn_lib = [
+                            '/opt/local/lib',                       # Darwin - darwin ports
+                            '/sw/lib',                              # Darwin - Fink
+                            '/usr/lib',                             # typical Linux
+                            '/usr/local/lib',                       # typical *BSD
+                            ]
+            self._find_paths_apr_inc = [
+                            '/opt/local/include/apr-1',             # Darwin - darwin ports
+                            '/sw/include/apr-1',                    # Darwin - fink
+                            '/usr/include/apr-1',                   # typical Linux
+                            '/usr/local/apr/include/apr-1',         # Mac OS X www.metissian.com
+                            ]
+            self._find_paths_apr_util_inc = self._find_paths_apr_inc
+            self._find_paths_apr_lib = [
+                            '/opt/local/lib',                       # Darwin - darwin ports
+                            '/sw/lib',                              # Darwin - fink
+                            '/usr/lib64',                           # typical 64bit Linux
+                            '/usr/lib',                             # typical Linux
+                            '/usr/local/lib64',                     # typical 64bit Linux
+                            '/usr/local/lib',                       # typical *BSD
+                            '/usr/local/apr/lib',                   # Mac OS X www.metissian.com
+                            '/usr/pkg/lib',                         # netbsd
+                            ]
 
         self._completeInit()
 
     def get_lib_name_for_platform( self, libname ):
-        return '%s.dylib' % (libname,)
+        # With Xcode 7.x no dylib in the SDK only the tbd files
+        return ['%s.dylib' % (libname,), '%s.tbd' % (libname,)]
 
     def setupUtilities( self ):
         self._addVar( 'CCCFLAGS',
@@ -1039,21 +1109,38 @@ class MacOsxCompilerGCC(CompilerGCC):
         self._addVar( 'LDEXE',          '%(CCC)s -g' )
 
     def setupPySvn( self ):
+        # Support building in a virtualenv.
+        #
+        # In a virtualenv on a Mac, sys.exec_prefix will be a relative path
+        # pointing to the root of the virtualenv. That root will *not* contain
+        # a Python directory, as needed by PYTHON_FRAMEWORK below.
+        #
+        # Instead, we use sys.real_prefix, which is added when in a virtualenv.
+        # This will point to the Python framework that the virtualenv is based
+        # upon, which does contain the Python directory. It's equivalent to
+        # sys.exec_prefix outside of a virtualenv.
+        python_prefix = getattr( sys, 'real_prefix', sys.exec_prefix )
+
         self._pysvnModuleSetup()
         self._addVar( 'PYSVN_MODULE_BASENAME', self.pysvn_module_name )
 
         self._addVar( 'PYTHON_VERSION',     '%d.%d' % (sys.version_info[0], sys.version_info[1]) )
         self._addVar( 'PYTHON_DIR',         sys.exec_prefix )
-        self._addVar( 'PYTHON_FRAMEWORK',   os.path.join( sys.exec_prefix, 'Python' ) )
+        self._addVar( 'PYTHON_FRAMEWORK',   os.path.join( python_prefix, 'Python' ) )
         self._addVar( 'PYTHON_INC',         distutils.sysconfig.get_python_inc() )
 
         py_cflags_list = [
                     '-g',
-                    '-Wall -fPIC -fexceptions -frtti',
+                    '-Wall -fPIC',
                     '-I. -I%(APR_INC)s -I%(APU_INC)s -I%(SVN_INC)s',
                     '-DPYCXX_PYTHON_2TO3 -I%(PYCXX)s -I%(PYCXX_SRC)s -I%(PYTHON_INC)s',
                     '-D%(DEBUG)s',
                     ]
+
+        if self.pycxx_version >= (7, 0, 0):
+            # PYSVN uses PYCXX in backward compat mode
+            py_cflags_list.append( r'-DPYCXX_6_2_COMPATIBILITY=1' )
+
         for define, value in self.py_module_defines:
             py_cflags_list.append( '-D%s=%s' % (define, value) )
 
@@ -1061,7 +1148,7 @@ class MacOsxCompilerGCC(CompilerGCC):
 
         py_ld_libs = [
                 '-L%(SVN_LIB)s',
-                '-L/usr/lib',
+                '-L%(APR_LIB)s',
                 '-lsvn_client-1',
                 '-lsvn_repos-1',
                 '-lsvn_wc-1',
@@ -1069,12 +1156,23 @@ class MacOsxCompilerGCC(CompilerGCC):
                 '-lsvn_subr-1',
                 '-lsvn_diff-1',
                 '-lapr-1',
-                '-lssl',
                 ]
 
-        self._addVar( 'CCCFLAGS', ' '.join( py_cflags_list ) )
+        self._addVar( 'CCFLAGS', ' '.join( py_cflags_list ) )
+        self._addVar( 'CCCFLAGS', ' '.join( ['-fexceptions -frtti'] + py_cflags_list ) )
         self._addVar( 'LDLIBS', ' '.join( py_ld_libs ) )
-        self._addVar( 'LDSHARED',       '%(CCC)s -bundle -g '
+
+        if self.options.hasOption( '--link-python-framework-via-dynamic-lookup' ):
+            # preferred link method on homebrew for pysvn
+            self._addVar( 'LDSHARED',   '%(CCC)s -bundle -g '
+                                        '-framework System '
+                                        '-framework CoreFoundation '
+                                        '-framework Kerberos '
+                                        '-framework Security '
+                                        '-undefined dynamic_lookup '
+                                        '%(LDLIBS)s' )
+        else:
+            self._addVar( 'LDSHARED',   '%(CCC)s -bundle -g '
                                         '-framework System '
                                         '%(PYTHON_FRAMEWORK)s '
                                         '-framework CoreFoundation '
@@ -1087,13 +1185,15 @@ class UnixCompilerGCC(CompilerGCC):
         CompilerGCC.__init__( self, setup )
 
         self._find_paths_pycxx_dir = [
-                        '../Import/pycxx-%d.%d.%d' % pycxx_version,
-                        distutils.sysconfig.get_python_inc() # typical Linux
+                        distutils.sysconfig.get_python_inc(), # typical Linux
+                        '/usr/include'
                         ]
         self._find_paths_pycxx_src = [
                         '%(PYCXX)s/Src',
-                        '/usr/share/python%s/CXX' % distutils.sysconfig.get_python_version() # typical Linux
+                        '/usr/share/python%s/CXX' % distutils.sysconfig.get_python_version(), # typical Linux
+                        '/usr/src/CXX'
                         ]
+
         self._find_paths_svn_inc = [
                         '/usr/include/subversion-1',            # typical Linux
                         '/usr/local/include/subversion-1',      # typical *BSD
@@ -1110,9 +1210,11 @@ class UnixCompilerGCC(CompilerGCC):
                         '/usr/local/lib64',                     # typical 64bit Linux
                         '/usr/local/lib',                       # typical *BSD
                         '/usr/pkg/lib',                         # netbsd
+                        '/usr/lib/x86_64-linux-gnu',            # debian/unbuntu
                         ]
         self._find_paths_apr_inc = [
                         '/usr/include/apr-1',                   # typical Linux
+                        '/usr/include/apr-1.0',                 # typical Linux
                         '/usr/local/apr/include/apr-1',         # Mac OS X www.metissian.com
                         '/usr/pkg/include/apr-1',               # netbsd
                         '/usr/include/apache2',                 # alternate Linux
@@ -1128,15 +1230,16 @@ class UnixCompilerGCC(CompilerGCC):
                         '/usr/local/lib',                       # typical *BSD
                         '/usr/local/apr/lib',                   # Mac OS X www.metissian.com
                         '/usr/pkg/lib',                         # netbsd
+                        '/usr/lib/x86_64-linux-gnu',            # debian/unbuntu
                         ]
 
         self._completeInit()
 
     def get_lib_name_for_platform( self, libname ):
         if self.setup.platform == 'cygwin':
-            return '%s.dll.a' % libname
+            return ['%s.dll.a' % libname]
         else:
-            return '%s.so' % libname
+            return ['%s.so' % libname]
 
     def setupUtilities( self ):
         self._addVar( 'CCCFLAGS',
@@ -1155,12 +1258,17 @@ class UnixCompilerGCC(CompilerGCC):
         self._addVar( 'PYTHON_ARCH_SPECIFIC_INC',     distutils.sysconfig.get_python_inc( True ) )
 
         py_cflags_list = [
-                    '-Wall -fPIC -fexceptions -frtti',
+                    '-Wall -fPIC',
                     '-I. -I%(APR_INC)s -I%(APU_INC)s -I%(SVN_INC)s',
                     '-DPYCXX_PYTHON_2TO3 -I%(PYCXX)s -I%(PYCXX_SRC)s -I%(PYTHON_INC)s',
                     '-I%(PYTHON_ARCH_SPECIFIC_INC)s',
                     '-D%(DEBUG)s',
                     ]
+
+        if self.pycxx_version >= (7, 0, 0):
+            # PYSVN uses PYCXX in backward compat mode
+            py_cflags_list.append( r'-DPYCXX_6_2_COMPATIBILITY=1' )
+
         for define, value in self.py_module_defines:
             py_cflags_list.append( '-D%s=%s' % (define, value) )
 
@@ -1174,7 +1282,8 @@ class UnixCompilerGCC(CompilerGCC):
             print( 'Info: Disable deprecated functions warnings' )
             py_cflags_list.append( '-Wno-deprecated-declarations' )
 
-        self._addVar( 'CCCFLAGS',   ' '.join( py_cflags_list ) )
+        self._addVar( 'CCFLAGS',    ' '.join( py_cflags_list ) )
+        self._addVar( 'CCCFLAGS',   ' '.join( py_cflags_list+['-fexceptions -frtti'] ) )
         self._addVar( 'LDLIBS',     ' '.join( self._getLdLibs() ) )
         self._addVar( 'LDSHARED',   '%(CCC)s -shared -g' )
 
@@ -1233,12 +1342,20 @@ class CygwinCompilerGCC(UnixCompilerGCC):
     def __init__( self, setup ):
         UnixCompilerGCC.__init__( self, setup )
 
+    def getPythonExtensionFileExt( self ):
+        return '.dll'
+
     def _getLdLibs( self ):
+        if sys.version_info[0] >= 3:
+            m = 'm'
+        else:
+            m = ''
+
         py_ld_libs = [
-                '-L%(svn_lib_dir)s',
-                '-L%(apr_lib_dir)s',
-                '-L/usr/lib/python%d.%d/config -lpython%d.%d.dll' %
-                    (sys.version_info[0], sys.version_info[1], sys.version_info[0], sys.version_info[1]),
+                '-L%(SVN_LIB)s',
+                '-L%(APR_LIB)s',
+                '-lpython%d.%d%s.dll' %
+                    (sys.version_info[0], sys.version_info[1], m),
                 '-lsvn_client-1',
                 '-lsvn_repos-1',
                 '-lsvn_subr-1',
@@ -1256,7 +1373,6 @@ class CygwinCompilerGCC(UnixCompilerGCC):
                 '-liconv',
                 '-lexpat',
                 '-lpthread',
-                '-lz',
                 ]
         return py_ld_libs
 
@@ -1333,7 +1449,7 @@ class Target:
             source.setDependent( self )
 
     def getTargetFilename( self ):
-        raise NotImplementedError( '%s.getTargetFilename' % self.__class__.__name__ )
+        raise NotImplementedError( 'getTargetFilename' )
 
     def generateMakefile( self ):
         if self.__generated:
@@ -1446,12 +1562,17 @@ class Source(Target):
         if basename.endswith( '.c' ):
             return self.compiler.platformFilename( self.compiler.expand( '%s%s' % (basename[:-len('.c')], obj_ext) ) )
 
-        raise SetupError( 'unknown source %r' % (self.src_filename,) )
+        raise SetupError( 'Unknown source %r' % (self.src_filename,) )
 
     def _generateMakefile( self ):
         debug( 'Source:0x%8.8x.generateMakefile() for %r' % (id(self), self.src_filename,) )
 
-        self.compiler.ruleCxx( self )
+        if self.src_filename.endswith( '.c' ):
+            self.compiler.ruleC( self )
+
+        else:
+            self.compiler.ruleCxx( self )
+
         self.compiler.ruleClean( self.getTargetFilename() )
 
 class PysvnSvnErrorsPy(Source):
